@@ -15,9 +15,7 @@
 #
 # You should have received a copy of the GNU Lesser General Public License along
 # with CubicWeb.  If not, see <http://www.gnu.org/licenses/>.
-"""unit tests for module cubicweb.schema
-
-"""
+"""unit tests for module cubicweb.schema"""
 
 import sys
 from os.path import join, isabs, basename, dirname
