@@ -129,8 +129,9 @@ class PrimaryView(EntityView):
         display_attributes = []
         for rschema, _, role, dispctrl in self._section_def(entity, 'attributes'):
             vid = dispctrl.get('vid', 'reledit')
-            if rschema.final or vid == 'reledit':
-                value = entity.view(vid, rtype=rschema.type, role=role)
+            if rschema.final or vid == 'reledit' or dispctrl.get('rtypevid'):
+                value = entity.view(vid, rtype=rschema.type, role=role,
+                                    initargs={'dispctrl': dispctrl})
             else:
                 rset = self._relation_rset(entity, rschema, role, dispctrl)
                 if rset:
@@ -153,6 +154,16 @@ class PrimaryView(EntityView):
 
     def render_entity_relations(self, entity):
         for rschema, tschemas, role, dispctrl in self._section_def(entity, 'relations'):
+            if rschema.final or dispctrl.get('rtypevid'):
+                self.w(u'<div class="section">')
+                label = self._rel_label(entity, rschema, role, dispctrl)
+                if label:
+                    self.w(u'<h4>%s</h4>' % label)
+                vid = dispctrl.get('vid', 'reledit')
+                entity.view(vid, rtype=rschema.type, role=role, w=self.w,
+                            initargs={'dispctrl': dispctrl})
+                self.w(u'</div>')
+                continue
             rset = self._relation_rset(entity, rschema, role, dispctrl)
             if rset:
                 try:
@@ -260,6 +271,20 @@ class PrimaryView(EntityView):
             label = display_name(self._cw, rschema.type, role)
         self.field(label, value, show_label=showlabel, tr=False, table=table)
 
+    def _rel_label(self, entity, rschema, role, dispctrl):
+        if rschema.final:
+            showlabel = dispctrl.get('showlabel', self.show_attr_label)
+        else:
+            showlabel = dispctrl.get('showlabel', self.show_rel_label)
+        if showlabel:
+            if dispctrl.get('label'):
+                label = self._cw._(dispctrl['label'])
+            else:
+                label = display_name(self._cw, rschema.type, role,
+                                     context=entity.__regid__)
+            return label
+        return u''
+
 
 class RelatedView(EntityView):
     __regid__ = 'autolimited'
